/** \file 2D_Object2D.h
    A 2D World Object.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_2D_OBJECT2D
#define H_2D_OBJECT2D

#include <WO_Object.h>
#include <2D_Types.h>
#include <CON_Math2D.h>

class World2D;

/** 2D World Object.  Connects the concepts of a 2D World and the bitmap.
    Provides movement and collision detection for sprites.
*/
class Object2D : public Object, public SpriteSelector
{
public:
  /** Construct a 2D Object in a specified world, with a certain parent.
      Using NULL for a parent causes the World to be the object's parent.
  */
  Object2D(World2D* W, Object2D* Parent);

  /** Destruct the object. */
  ~Object2D();

  /** Calculate the object's global position.  The global position
      is relative to the world, whereas the object's (local) position
      is relative to its parent.
      Non-zero AllTheWay means that the calculation will recursively
      go up the tree all the way to the root.  Otherwise, the
      calculation will only go up one level.
  */
  virtual long calcGlobalPosition(int AllTheWay=0);

  /** Render the sprite to the current 2D view. */
  virtual long render();

  /** Move/Change the object state.  Defaults to nothing. */
  virtual long advance(float Fraction);

  /** Add an object as a son.  Any son must also be a Object2D. */
  virtual long add(Object* O);

  /** Check for collision with this 2D object. */
  virtual long collision(Object2D* O);

  /** Set the local (relative to parent) position. */
  long       setPosition(fVector2D P);

  /** Retrieve the object's global position.  The global position
      is relative to the world, whereas the object's (local) position
      is relative to its parent.
  */
  Vector2D&  getGlobalPosition();

  /** Retrieve the object's local position. */
  fVector2D& getPosition();

  /** Retrieve the mid point of the object.  
      This point is relative to the parent.
  */
  fVector2D  getMiddle();

  /** Retrieve the sprite associated with this object. */
  virtual Sprite* getSprite();

protected:
  fVector2D        m_Position;
  Vector2D         m_GlobalPosition;
  SpriteSelector  *m_SpriteSelector;
};

#endif // H_2D_OBJECT2D