/** \file 3D_Object3D.h
    3D World Object.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_3D_OBJECT3D
#define H_3D_OBJECT3D

#include <DS_SpecContainable.h>
#include <CON_3D.h>
#include <CON_Sound.h>
#include <3D_World.h>

class World3D;

/** 3D World Object.  Connects the concepts of a 3D World and the Mesh.
    Provides movement and collision detection for Models. */
class Object3D : public Object
{
public:
  /** Construct a world object.  Requires the world the object is in,
      the mesh that is associated with the object and the object's
      parent in the objects hierarchy.  The parent defaults to the 
      world in case the parameter is NULL.
  */
  Object3D(World3D* W, Mesh* O, Object3D* Parent);
  ~Object3D();

  /** Main rendering method.  Renders the object to the current active camera. */
  long render();

  /** Main progress method.  The Fraction indicates how much progress should be done.
      For example: If this objects advances at speed 5 units per second,
                   and Fraction=0.2   then the object should advance 5*0.2=1 unit.
      This allows keeping a synchronized behaviour, regardless to frame rate. */
  long advance(float Fraction);

  /** Positions the 3D sounds the object has.  (Called by advance automatically) */
  long orientSounds();

  /** Attaches a soundclip to the object.  This must be dynamically allocated, 
      and the object will delete it automatically. */
  long startSound(SoundClip* SC, int Loop=1);

  /** Stops a soundclip that is currently playing.
      Note: Stopping the soundclip, causes it to be deleted. */
  long stopSound(SoundClip* SC);

  // The following 3 methods determine the behaviour of the object.
  // Override them to supply specific behaviour.

  /** Returns non-zero if you want this object to be checked for collisions
      Defaults to Yes */
  virtual long checkCollisions();

  /** Returns non-zero if this object should be rendered.
      Defaults to Yes */
  virtual long visible();

  /** Returns non-zero if it should be added to the radar object, if any.
      Defaults to Yes */
  virtual long radarDetectable();

  /** Provides behaviour in case of a collision with another object.
      Defaults to nothing. */
  virtual long collision(Object3D* O);

  friend class World3DImpl;
  friend class CollisionDetection3D;
protected:
  Mesh*        m_Mesh;
  World3D*     m_World;
  Vector       m_Sounds;
};

/** An object which moves the camera according to its movements */
class Viewer : public Object3D 
{
public:
  /** Construct a 3D Object and associate a camera to it. */
  Viewer(World3D* W, Mesh* O, Camera *Cam);
  ~Viewer();

  /** Overrides the Object3D default behaviour, and renders nothing. */
  long render();

  /** Moves the camera object to where this object is. */
  long advance(float Fraction);

  /** Returns NO, because we never see the viewer's mesh. */
  virtual long visible();

  /** Returns NO, because we don't detect ourselves. */
  virtual long radarDetectable();

protected:
  Camera* m_Cam;
};

#endif // H_3D_OBJECT3D