/** \file CON_3D.h
    3D Rendering interfaces: Screen3D, Camera, Mesh, Light

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.
Permission is granted to use and copy this file for non-commercial use only.
Please contact the author concerning commercial usage.
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_CON_3D
#define H_CON_3D

#include <istream.h>
#include <CON_Basic.h>
#include <CON_Misc.h>
#include <CON_Math3D.h>

// Forward declarations
class SoundBuffer;

/**
A Mesh of triangles that represent an object in 3D Space.
The interface provides methods for: creating the mesh, moving it in 3D space and rendering it.
*/
class Mesh : public Interface, public Transformable
{
public:
  /** Render the object to the current scene using the
      transformation given by transform.  If transform == NULL,
      the Mesh's own internal transform will be used. */
  virtual long render(Matrix3D* transform=NULL) = 0;

  /** Returns status of object (0=ok, else error) */
  virtual long status() = 0;

  /** Locate a 3D sound buffer in this object's location */
  virtual long orientSound(SoundBuffer* S) = 0;

  /** Check for collision with this object
      Returns a non-zero value if a collision is detected. If transform ==
      NULL, both this and Mesh O are to be transformed by their own internal
      transformations; otherwise, the given transform is applied to O and is
      assumed to be relative to this (no transform is applied to this). */
  virtual long collision(Mesh* O, Matrix3D* transform=NULL) = 0;


  /** Should be called from main world advance if blinking materials are used. */
  virtual long advance(float Fraction) = 0;

  /** Set a texture for this object */
  virtual long setTexture(const char* Name, void* Texture) = 0;

/** section: Historical interface functions for backwards compatibility.  
    Use functions inherited from Transformable in new code. **/

  /** Set the object's position */
  virtual long setPosition(float x, float y, float z) = 0;

  /** Move the object relative to its current position */
  virtual long relocate(float dx, float dy, float dz) = 0;

  /** Rotate around any or all axes (angles in radians) */
  virtual long rotate(float ax, float ay, float az) = 0;

  /** Set the position and orientation according to this object */
  virtual long align(Mesh* O) = 0;

  /** Get direction vector, scaled to Size */
  virtual Vector3D direction(float Size=1.0f) = 0;

  /** Get the up direction vector, scaled to Size */
  virtual Vector3D direction_up(float Size=1.0f) = 0;

/** end of historical section */

  /** Get the object's rough radius, (non-optimized bounding sphere) */
  virtual float    getRadius() = 0;

  /** Set the object's radius, other than its actual one. */
  virtual long     setRadius(float Radius) = 0;

  /** Exports the object to a text .X file */
  virtual long exportXFile(ostream& os) = 0;

  /** Exports the model's data to the supplied areas.
      Specify NULL for any parameter to skip retrieval of that info.
      Vertices Array size should be  5 * VerticesNumber
      Faces Array size should be  4 * FacesNumber
      Materials Array size should be 17 * MaterialsNumber */
  virtual long exportData(float* Vertices, int* VerticesNumber,
                          short* Faces, int* FacesNumber, 
                          float* Materials, int* MaterialsNumber) = 0;

  /** Category: Setup Methods which are used to construct objects from scratch.
      These need not be used when loading objects from .X files. */

  /** Add a vertex (point) at the specified position, optionally specifying texture coordinates */
  virtual long addVertex(float x, float y, float z, float u=0.0f, float v=0.0f) = 0;

  /** Get how many vertices have been entered so far (useful for independent subobjects) */
  virtual int  getSetupVerticesNumber() = 0;

  /** In case texture coords are loaded separately that the world coords, 
      it is possible to set it later.  Vertex index is required. */
  virtual long setVertexTextureCoords(int index, float u=0.0f, float v=0.0f) = 0;

  /** Add a triangle with a,b,c specifying the vertices that form this triangle
      and the material name specifies the material it is made of.
      Note: The index order determines its visible side (Clockwise) */
  virtual long addFace(short a, short b, short c, const char* MaterialName) = 0;

  /** Same as the previous, but uses the index of the material instead,
      In case you don't have its name */
  virtual long addFace(short a, short b, short c, int MaterialNumber) = 0;

  /** Add a material with the key name and the list of colors that make the material. */
  virtual long addMaterial(const char* Name, ushort Diffuse, ushort Specular, 
                           ushort Emissive, float Power, void* Texture=NULL) = 0;

  /** Add a material, used internally by the .X import */
  virtual long addMaterial(const char* Name, void* MaterialDesc, void* Texture=NULL) = 0;

  /** This method can be used to calculate normals for models that have unordered 
      faces.  It will copy itself onto 'O' and update the normals.
      O must be a newly created object with no data. */
  virtual long calculateNormals(Mesh* O, ProgressMonitor* PM=NULL) = 0;

  /** This method creates a copy of the current object, but reduces the polygon
      count approximately according to the reduction ratio.
      This is used for creating different Levels Of Details  (LODs)
      O must be a newly created object with no data. */
  virtual long createReducedObject(Mesh* O, float Reduction=0.75f, ProgressMonitor* PM=NULL) = 0;

  /** Setup complete, prepare for rendering.
      This function must be run in order to process all data entered by the previous functions.
      The 'JoinVertices' parameter determines if nearby vertices should be joined into one.
      nearby is determined if the square distance is less than this parameter.
      Therefore, -1.0 means 'No' */
  virtual long finalize(float JoinVertices=-1.0f) = 0;
  virtual long finalize(float JoinVertices, float GouraudThreshold) = 0;

};

/*
// The following subclass is a simple billboard of a round disc.
class DLLExport PlainPlanet3D : public Mesh
{
public:
  // Specify position size and color, and all setup routines
  // are called internally.
  PlainPlanet3D(Screen3D* S, float x, float y, float z, float Size, int Red, int Green, int Blue);
  // Internal constructor, used by subclasses.
  PlainPlanet3D(MeshImpl* O, float x, float y, float z, float Size, int Red, int Green, int Blue);
};

// A real sphere covered by a texture
class DLLExport Planet3D : public Mesh
{
public:
  // Specify position, size, and texture fetched from the Texture Cache
  Planet3D(Screen3D* S, float x, float y, float z, float Size, void* Texture);
};

class DLLExport Splinter : public Mesh
{
public:
  // A splinter, which is a single triangle object, which has a constant velocity,
  // and a constant rotation 
  Splinter(Screen3D* S, float x, float y, float z);
};

*/

/**
3D Rendering control interface.  Must be created to enable 3D Rendering.
*/
class Screen3D : public SystemInterface
{
public:
  /** Return the D3D Device pointer for creating special effects.
      Cast the return value to LPDIRECT3DDEVICE3
  */
  virtual void* getD3DDevice() = 0;
  /** Returns a non zero value if an error has occured. */
  virtual long status() = 0;
  /** Must be called before any 3D Rendering */
  virtual long beginScene() = 0;
  /** Renders an object */
  virtual long renderObject(Mesh* O) = 0;
  /** Must be called after all rendering is done, and before flipping. */
  virtual long endScene() = 0;
  /** Can be called to override the default ambient lighting. */
  virtual long setAmbientLight(unsigned long Color) = 0;
};

/** Generic light */
class Light : public Interface
{
public:
  /** Returns non-zero if an error has occurred. */
  virtual long status() = 0;
};

/**
A camera that gives a view point of the 3D world.
Only one camera can be active at a time, due to a tie with
a virtual microphone, giving the 3D sounds of the world.
*/
class Camera : public Interface
{
public:
  /** Clear the view screen, should be called before rendering a scene
      Color default of -1 uses the background color as set by setBackgroundColor. 
      The clear can do one or two (or none) things, clearing the view buffer
      and/or the zbuffer. */
  virtual long clear(long Color=-1, int ClearView=1, int ClearZBuf=1) = 0;

  /** Called prior to rendering for internal special effects. */
  virtual long preRender() = 0;

  /** Called after rendering for internal special effects. */
  virtual long postRender() = 0;

  /** Set the viewable area position and dimensions in 2D screen coordinates */
  virtual long setViewable(int x, int y, int width, int height) = 0;

  /** Set the position of the camera in the 3D World */
  virtual long setPosition(float x, float y, float z) = 0;

  /** Set the direction that the camera is pointed to. */
  virtual long setDirection(float tox, float toy, float toz,
                            float upx, float upy, float upz) = 0;

  /** Set the camera velocity.  This currently is used for audio doppler effects */
  virtual long setVelocity(float v) = 0;

  /** Add stars to a space scene, to be rendered in the preRender stage. */
  virtual long setStars(int Number=1000) = 0;

  /** Add a light to the scene.  Light objects are not maintained by
      the camera, and should be allocated and freed by the application */
  virtual long addLight(Light* L) = 0;

  /** remove a light from the scene. */
  virtual long removeLight(Light* L) = 0;

  /** Set the camera to show what the object 'sees' */
  virtual long follow(const Transformable& t) = 0;

  /** Activate the camera. */
  virtual long activate() = 0;

  /** Returns non zero if an error has occured. */
  virtual long status() = 0;

  /** Gets the camera's position in the 3D World  */
  virtual Vector3D getPosition() = 0;

  /** Gets the position of an object relative to the camera */
  virtual Vector3D getObjectPosition(Mesh* O) = 0;

  /** Gets the current default color used to clear the view buffer. */
  virtual ushort   getBackgroundColor() = 0;

  /** Sets the current default color used to clear the view buffer. */
  virtual void     setBackgroundColor(ushort Color) = 0;
};


#ifndef FUNC_3D
#define FUNC_3D
////////////////////////////////////////////////////////////
// 3D Functions
////////////////////////////////////////////////////////////

#include <CON_Resource.h>
#include <CON_Texture.h>

/** Initialization of 3D subsystem.  
    Must be performed prior to using any other 3D functions.
    Will be released automatically. 
*/
DLLExport Screen3D*  initScreen3D();

/** Create a new camera object.
*/
DLLExport Camera*    newCamera();

/** Create a new mesh object.
*/
DLLExport Mesh*      newMesh();

/** Create a new light, which is a parallel point (like Sun Light) with a specified color.
    Coordinates specify the initial position for the light.
*/
DLLExport Light*     newSunLight(float x, float y, float z, float Red=0.7f, float Green=0.9f, float Blue=0.7f);

/** Create a new BillBoard.  This is a mesh that always faces the camera to allow 2D
    art in the 3D World.
*/
DLLExport Mesh*      newBillBoard();

/** Create a new BillBoard Sphere.  This is a a bill board that is simply a circle.
    Coordinates specify the initial position for the mesh.
*/
DLLExport Mesh*      newBillBoardSphere(float x, float y, float z, float Size, int Red, int Green, int Blue);

/** Create a new Sphere.  This is not a bill board, but a real 3D Sphere.
    Coordinates specify the initial position for the mesh.
    Size is the radius of the sphere.
    A texture can be applied and will wrap the sphere.
*/
DLLExport Mesh*      newSphere(float x, float y, float z, float Size, void* Texture, int Inv=0);


/** Initializes an internal texture cache.  
    If RS is NULL, actual files will be used to load textures. 
    Setting Destroy to non zero will release the resource stream
    when the texture cache is released.
*/
DLLExport void          initTextureCache(ResourceStream* RS=NULL, int Destroy=0);

/** Creates a user TextureCache.  
    It must be release()d when it is no longer needed.
    Setting Destroy to non zero will release the resource stream
    when the texture cache is released.
*/
DLLExport TextureCache* newTextureCache(ResourceStream* RS, int Destroy=0);

/** Loads the object from a .X file. name specifies file name */
DLLExport long loadXFile(const char* Name, Mesh* O, TextureCache* TC=NULL, float Scale=1.0f);



#endif // FUNC_3D


#endif // H_CON_3D