/** \file CON_Exports.h
    Definitions of the DLL's interface. 

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_CON_EXPORTS
#define H_CON_EXPORTS

#include <CON_Console.h>

////////////////////////////////////////////////////////////
// Generic functions.
////////////////////////////////////////////////////////////
#ifndef FUNC_GENERIC
#define FUNC_GENERIC


/** \relates Console
    Returns the version of the library. */
DLLExport float    getVersion();

/** \relates Console
    Returns the Fraction frame time needed for a world's advance. */
DLLExport float    getFrameTime();

/** \relates Console
    Returns the Console object pointer. */
DLLExport Console* getConsole();


#endif // FUNC_GENERIC


#ifndef FUNC_SPECIAL
#define FUNC_SPECIAL
////////////////////////////////////////////////////////////
// Special Purpose.
////////////////////////////////////////////////////////////

/** \relates Screen 
    Used to create a screen for console (non graphic) apps batch processing. 
    For a usual graphics game, it shouldn't be used.
*/
DLLExport Screen*  newWindowScreen(void* hWnd_, int Width, int Height, int NoAccel=0);


#endif // FUNC_SPECIAL
////////////////////////////////////////////////////////////
// Bitmap creation.
////////////////////////////////////////////////////////////
#ifndef FUNC_BITMAP
#define FUNC_BITMAP


/** \relates Drawable
    Creates a new bitmap with the specified size. */
DLLExport Bitmap* newBitmap(int Width, int Height, int NoAccel=0);

/** \relates Drawable
    Creates a copy of the given bitmap. */
DLLExport Bitmap* newBitmap(Bitmap& BM);

/** \relates Drawable
    Creates a bitmap and loads it from the input stream */
DLLExport Bitmap* newBitmap(istream& is, int Width, int Height, int NoAccel=0);

/** \relates Drawable
    Creates a bitmap and loads it from the input stream */
DLLExport Bitmap* newBitmap(istream& is, int NoAccel=0);

/** \relates Drawable
    Creates a bitmap and loads it from a *.BMP file */
DLLExport Bitmap* loadBMP(const char* Filename);

/** Old version given for compatibility. */
inline Bitmap* loadBitmap(Screen* S, istream& is)
{
  return newBitmap(is,S->getNoAccel());
}


#endif // FUNC_BITMAP
////////////////////////////////////////////////////////////
// 3D Functions
////////////////////////////////////////////////////////////
#ifndef FUNC_3D
#define FUNC_3D


/** \relates Screen3D
    Initialization of 3D subsystem.  
    Must be performed prior to using any other 3D functions.
    Will be released automatically. 
*/
DLLExport Screen3D*  initScreen3D();

/** \relates Camera
    Create a new camera object.
*/
DLLExport Camera*    newCamera();

/** \relates Mesh
    Create a new mesh object.
*/
DLLExport Mesh*      newMesh();

/** \relates Light
    Create a new light, which is a parallel point (like Sun Light) with a specified color.
    Coordinates specify the initial position for the light.
*/
DLLExport Light*     newSunLight(float x, float y, float z, float Red=0.7f, float Green=0.9f, float Blue=0.7f);

/** \relates Mesh
    Create a new BillBoard.  This is a mesh that always faces the camera to allow 2D
    art in the 3D World.
*/
DLLExport Mesh*      newBillBoard();

/** \relates Mesh
    Create a new BillBoard Sphere.  This is a a bill board that is simply a circle.
    Coordinates specify the initial position for the mesh.
*/
DLLExport Mesh*      newBillBoardSphere(float x, float y, float z, float Size, int Red, int Green, int Blue);

/** \relates Mesh
    Create a new Sphere.  This is not a bill board, but a real 3D Sphere.
    Coordinates specify the initial position for the mesh.
    Size is the radius of the sphere.
    A texture can be applied and will wrap the sphere.
*/
DLLExport Mesh*      newSphere(float x, float y, float z, float Size, void* Texture, int Inv=0);


/** \relates TextureCache
    Initializes an internal texture cache.  
    If RS is NULL, actual files will be used to load textures. 
    Setting Destroy to non zero will release the resource stream
    when the texture cache is released.
*/
DLLExport void          initTextureCache(ResourceStream* RS=NULL, int Destroy=0);

/** \relates TextureCache
    Creates a user TextureCache.  
    It must be release()d when it is no longer needed.
    Setting Destroy to non zero will release the resource stream
    when the texture cache is released.
*/
DLLExport TextureCache* newTextureCache(ResourceStream* RS=NULL, int Destroy=0);

/** \relates Mesh
    Loads the object from a .X file. name specifies file name */
DLLExport long loadXFile(const char* Name, Mesh* O, TextureCache* TC=NULL, float Scale=1.0f);



#endif // FUNC_3D
////////////////////////////////////////////////////////////
// Sound Functions
////////////////////////////////////////////////////////////
#ifndef FUNC_SOUND
#define FUNC_SOUND


/** \relates SoundBuffer
    Create a new sound clip from raw data.  Size is in bytes. */
DLLExport SoundClip* newSoundClip(char* AudioData, int Size,    int Freq=44100, int Bits=16, int Channels=2, int HW=SoundAccel);

/** \relates SoundBuffer
    Create a new sound clip from raw data (taken from a binary stream).  Size is in bytes. */
DLLExport SoundClip* newSoundClip(istream* AudioData, int Size, int Freq=44100, int Bits=16, int Channels=2, int HW=SoundAccel);

/** \relates SoundBuffer
    Create a new sound stream from raw data.  
    Stream must remain valid until this object is destroyed.
    Data will be streamed to the sound card continuously.
*/
DLLExport SoundStream* newSoundStream(ResourceStream* RS, istream* AudioData, int Freq=44100, int Bits=16, int Channels=2, int HW=0);

/** \relates SoundBuffer
    Loads a wave from a binary input stream */
DLLExport SoundClip* loadWaveFile(istream* is, int HW=SoundAccel);

/** \relates SoundBuffer
    Loads a wave from the default resource stream (is it's set), or a file. */
DLLExport SoundClip* loadWaveFile(const char* Name, int HW=SoundAccel);

/** Old version.  Provided for compatibility. */
DLLExport SoundClip* loadWaveFile(Sound* S, istream& is, int HW);

/** Old version.  Provided for compatibility. */
DLLExport SoundClip* loadWaveFile(Sound* S, const char* Name, int HW);

/** \relates SoundBuffer
    Open a large wave file for streaming.
    The ResourceStream must remain alive until the SoundStream is released.
    It will be used to release the data input stream.
*/
DLLExport SoundStream* streamWaveFile(ResourceStream* RS, const char* Name, int HW=0);


#endif // FUNC_SOUND
////////////////////////////////////////////////////////////
// Keyboard Functions
////////////////////////////////////////////////////////////
#ifndef FUNC_KEYBOARD
#define FUNC_KEYBOARD


/** \relates Keyboard
    Translate key ID into the character they represent on the keyboard. */
char DLLExport getKeyChar(int Key);


#endif // FUNC_KEYBOARD
////////////////////////////////////////////////////////////
// ResourceStream Functions
////////////////////////////////////////////////////////////
#ifndef FUNC_RESOURCE
#define FUNC_RESOURCE


/** \relates ResourceStream
    Create a new ResourceStream that uses disk files directly. */
DLLExport ResourceStream* newResourceStream();

/** \relates ResourceStream
    Create a new ResourceStream for writing. 
    Requires stream of the output resource file 
*/
DLLExport ResourceStream* newResourceStream(ofstream* os);

/** \relates ResourceStream
    Create a new ResourceStream for reading (Create=0) or writing (Create=1).  
    Will open a disk file by the given name.
*/
DLLExport ResourceStream* newResourceStream(const char* Name, int Create=0);

/** \relates ResourceStream
    Set a resource stream as the system default resource stream.
    It will be released when the program terminates, or when setting 
    a different stream as the system default resource stream.
*/
DLLExport ResourceStream* setDefaultResourceStream(ResourceStream* RS);


#endif // FUNC_RESOURCE
////////////////////////////////////////////////////////////
// Debugging Functions
////////////////////////////////////////////////////////////
#ifndef FUNC_DEBUGGING
#define FUNC_DEBUGGING


/** \relates Console
    Sets the level of debugging information. 
    Defaults to 0 which outputs no log.
    Setting to a different value will create a log file 'libcon.log'
    That will be filled with printouts.
*/
DLLExport void setDebugLevel(int Level);

/** \relates Console
    Enable a mechanizm that tracks allocated interfaces.
    If used, it must be called before creating any interfaces.
    Diagnosis will be printed (If the debug level is set) when
    the program terminates.  The printouts will contain the 
    types of interfaces that were not released.
*/
DLLExport void enableDiagnostics();

/** \relates Console
    Can be used to append the log file with an error message. 
    rc is a return code, or any informational number.
*/
DLLExport void ReportError(long rc, const char* Message);

/** \relates Console
    Can be used to append the log file with an warning message. 
    rc is a return code, or any informational number.
*/
DLLExport void ReportWarning(long rc, const char* Message, int MinDebug=1);

/** \relates Console
    Can be used to append the log file with an informational message. 
*/
DLLExport void ReportInfo(const char* Message, int MinDebug=1);


#endif // FUNC_DEBUGGING
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

#endif // H_CON_EXPORTS