/** \file CON_Sound.h
    Sound Objects (Clip & Stream) 

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_CON_SOUND
#define H_CON_SOUND

#include <CON_3D.h>

/** Main sound object.  It is created by Console */
class Sound : public SystemInterface
{
public:
  /** Volume must be between -10000 (silence) and 0 (Full) */
  virtual long setVolume(int Volume) = 0;

  /** Returns non zero if an error has occured. */
  virtual long status() = 0;
};


/** Flags passed in the HW parameter when creating sounds
*/
enum {
  SoundAccel=1,  // Enable hardware acceleration
  Sound3D=2      // Enable 3D sound
};

/** Generic SoundBuffer class implements the sound generics
    3D methods are not a must.  Standard mono / stereo can be played too. */
class SoundBuffer : public Interface
{
public:
  /** Sets the Position in the 3D World. */
  virtual long setPosition(float x, float y, float z) = 0;

  /** Sets the direction and speed in the 3D World. 
      The speed will not affect the sound's position, and will be used
      only for making doppler effects. */
  virtual long setDirection(float x, float y, float z, float Speed) = 0;

  /** Set 3D Buffer parameters */
  virtual long set3DParameters(float MinDistance=15.0f, float MaxDistance=5000.0f, 
                               float ConeInside=60, float ConeOutside=360,
                               int   VolumeOutside=-5000) = 0;

  /** Determines whether this sound is a 3D sound or not 
      Note: if a buffer was not created with the Sound3D flag
            it cannot be set to 3D mode.*/
  virtual long set3DMode(int Mode) = 0;

  /** Detaches the sound buffer, and lets it play until it stops.
      Sound buffer will then be deleted!!! 
      Not recommended for loop playing sound clips */
  virtual long destroyOnStop() = 0;

  /** Returns non zero if the buffer is currently playing. */
  virtual int  isPlaying() const = 0;

  /** Volume must be between -10000 (silence) and 0 (Full) */
  virtual long setVolume(int Volume) = 0;

  /** Play the sound. If Loop is non zero, it will play over and over */
  virtual long play(int Loop) = 0;

  /** Stop playing */
  virtual long stop() = 0;

  /** Returns non zero if an error occured. */
  virtual long status() const = 0;

};

typedef SoundBuffer SoundClip;
typedef SoundBuffer SoundStream;


////////////////////////////////////////////////////////////
// Sound Functions
////////////////////////////////////////////////////////////
#ifndef FUNC_SOUND
#define FUNC_SOUND


/** Create a new sound clip from raw data.  Size is in bytes. */
DLLExport SoundClip* newSoundClip(char* AudioData, int Size,    int Freq=44100, int Bits=16, int Channels=2, int HW=SoundAccel);

/** Create a new sound clip from raw data (taken from a binary stream).  Size is in bytes. */
DLLExport SoundClip* newSoundClip(istream* AudioData, int Size, int Freq=44100, int Bits=16, int Channels=2, int HW=SoundAccel);

/** Create a new sound stream from raw data.  
    Stream must remain valid until this object is destroyed.
    Data will be streamed to the sound card continuously.
*/
DLLExport SoundStream* newSoundStream(ResourceStream* RS, istream* AudioData, int Freq=44100, int Bits=16, int Channels=2, int HW=0);

/** Loads a wave from a binary input stream */
DLLExport SoundClip* loadWaveFile(istream* is, int HW=SoundAccel);

/** Loads a wave from the default resource stream (is it's set), or a file. */
DLLExport SoundClip* loadWaveFile(const char* Name, int HW=SoundAccel);

/** Old version.  Provided for compatibility. */
DLLExport SoundClip* loadWaveFile(Sound* S, istream& is, int HW);

/** Old version.  Provided for compatibility. */
DLLExport SoundClip* loadWaveFile(Sound* S, const char* Name, int HW);

/** Open a large wave file for streaming.
    The ResourceStream must remain alive until the SoundStream is released.
    It will be used to release the data input stream.
*/
DLLExport SoundStream* streamWaveFile(ResourceStream* RS, const char* Name, int HW=0);


#endif // FUNC_SOUND


#endif // H_CON_SOUND