/** \file DS_Enumeration.h
    Linear representation for any data structure.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/

////////////////////////////////////////////////////////////
// Enumeration
//
// Purpose  - Create a serial list reference to another data structure.
// Creation - Enumeration objects are returned by other data structures,
//            upon request for an enumeration of their elements.
//
////////////////////////////////////////////////////////////

#ifndef H_DS_ENUMERATION
#define H_DS_ENUMERATION

class Enumeration;  // Defined here, for use as a return type in Vector.

#include <DS_Containable.h>
#include <DS_Exceptions.h>

class NotInitialized : public ConsistencyException
{
public:
  NotInitialized() : ConsistencyException("Not Initialized")
  {}
};

class Vector; // Forward declaration

/** This class provides a serial list of elements.
    This is especially useful for non-vector containers
    An instance of this class is returned by many containers methods.

    NotInitialized is thrown if the enumeration was created with no data.
    All enumerations created using the elements() method of containers
    contain data, regardless to the data in the container. */
class Enumeration
{
public:
  /** Construct an empty Enumeration.  operator= must be used before
      using any of the Enumeration's methods. */
  Enumeration();

  /** Construct an Enumeration with the elements copied from a vector. */
  Enumeration(Vector* V);

  /** Copy constructor. */
  Enumeration(Enumeration& E);

  /** Destructor */
  virtual ~Enumeration();

  /** Copies an Enumeration. */
  Enumeration& operator= (Enumeration& E);

  /** Returns a non-zero value if there are more elements to be read. */
  DS_BOOL hasMoreElements() const;

  /** Returns the next element  Throws IndexOutOfBounds if there are no more elements. */
  Data nextElement();

  /** Reset to read the elements again from the start. */
  void reset();

  /** Reverse the order of reading.  Reset must be applied afterwards to read from the start. */
  void reverse();

private:
  void loadElements(Vector& V);
  Vector *m_V;
  int     m_pos;
  int     m_dir;
};


#endif // H_DS_ENUMERATION
