/** \file DS_Hashtable.h
    Associative data structure.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_DS_HASHTABLE
#define H_DS_HASHTABLE

#include <DS_Containable.h>
#include <DS_Enumeration.h>
#include <DS_Container.h>

#ifndef BASE_HASHTABLE
  #define BASE_HASHTABLE void
#endif

/** A hashtable is a set of elements (unique elements)
    Each element is identified by its hash value (see Containable)
    Note: In non-destructive hashtables, elements must not be deleted
          before they are removed from the table.
    All methods that accept "Data" ( except put() )  do not require
    the actual item (this would be rediculous) but a containable
    that has the same hash value as the required item. */
class Hashtable : public Container
{
public:
  /** Construct an empty hash table. */
  Hashtable(unsigned Flags=0);

  /** Construct a copy of a hash table. */
  Hashtable(Hashtable& H);

  /** Destructor. */
  ~Hashtable();

  /** Copy a hash table. */
  Hashtable& operator= (Hashtable& H);

  /** Returns non-zero if the item is in the hash table. */
  DS_BOOL     contains(Data Item);

  /** Returns an enumeration of the hash table's elements. */
  Enumeration elements();

  /** Returns non-zero if there is no data in the hash table. */
  DS_BOOL     isEmpty();

  /** Insert a data item to the hash table. 
      If an item with the same key exists, the new one will overwrite
      the old one.
  */
  void        put(Data Item);

  /** Returns the number of elements in the hash table. */
  int         size() const;

  /** Retrieve a data item from the hash table.  The parameter must have
      the same key and return true when tested for equality. 
      For example, a Serial_Containable instance cannot be retrieved
      from a hashtable, without its actual pointer, because it only equals
      another Containable which has in the same memory address exactly.
  */
  Data        get(Data Item);

  /** Remove all elements from the hash table. */
  void        clear();

  /** Remove an item.  Item must conform to Hashtable::get rules. */
  void        remove(Data Item);

  /** Remove an element by internal order number. */
  void        removeElement(int i);

protected:
  BASE_HASHTABLE *m_Table;
  Vector         *m_elements;
};

#endif // H_DS_HASHTABLE
