/** \file DS_Vector.h
    Linear data structure

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.  
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_DS_VECTOR
#define H_DS_VECTOR

#include <DS_Container.h>
#include <DS_Containable.h>
#include <DS_Enumeration.h>

#ifndef BASE_VECTOR
  #define BASE_VECTOR void
#endif

class Vector : public Container
{
public:
  /** Construct an empty vector. */
               Vector(unsigned Flags=0);

  /** Construct an copy of a vector. */
               Vector(Vector& Copy);

  /** Destructor. */
               ~Vector();

  /** Copy a vector. */
  Vector&      operator= (Vector& Copy);

// Element Addition, set  

  /** Add an elelent at the end of the vector. */
  void         addElement(const Data Item);

  /** Insert an element at a specified position.
      This method pushes all following elements one forward.
      throws IndexOutOfBounds
  */
  void         insertElementAt(const Data Item, const int i);

  /** Change the data value of the element, without adding elements 
      throws IndexOutOfBounds
  */
  void         setElementAt(const Data Item, const int i);

  /** Relocate the item to the end of the vector. 
      throws IndexOutOfBounds
  */
  void         pushToEnd(const int i);

  /** Relocate the item to the beginning of the vector. 
      throws IndexOutOfBounds
  */
  void         pullToFront(const int i);

  /** Swap the elements in the two positions.
      throws IndexOutOfBounds
  */
  void         swapElemencts(const int i1, const int i2);


// Element Retrieval, get

  /** Retrieve an element at the specified index. 
      throws IndexOutOfBounds
  */
  Data         elementAt(const int i) const;

  /** Same as elementAt */
  Data         operator[] (const int i) const;

  /** Retrieve the item at position 0. 
      throws IndexOutOfBounds
  */
  Data         firstElement() const;

  /** Retrieve the item at position size()-1. 
      throws IndexOutOfBounds
  */
  Data         lastElement() const;

  /** Return an Enumeration of all elements.  This is a copy of the elements,
      so the vector can be modified while iterating through the Enumeration. 
  */
  Enumeration  elements();

// Element Removal

  /** Remove all items from the Vector. */
  void         removeAllElements();

  /** Remove the element at position size()-1 
      throws IndexOutOfBounds
  */
  void         removeLastElement();

  /** Remove the element at the specified position
      throws IndexOutOfBounds
  */
  void         removeElement(const int i);

// Indicators

  /** Number of elements in the container */
  int          size() const;

  /** Returns non-zero if there are no items in the vector. */
  DS_BOOL      isEmpty() const;

  /** Checks if this item is in the container. Returns non-zero if the item is in the vector. */
  int          contains(const Data Item) const;

  /** Searches serially for the item, returns -1 if not found. 
      Otherwise, returns the index of the item.
  */
  int          indexOf(const Data Item) const;

  /** Searches serially (from the end), returns -1 if not found.
      Otherwise, returns the index of the item.
  */
  int          lastIndexOf(const Data Item) const;

// Sort, Search

  /** Sort all items in the vector.  Uses Containable::lessThan to compare items. */
  void         Sort();

  /** Binary search, Assumes vector is sorted. */
  DS_BOOL      Find(Data& Item) const;

protected:
  BASE_VECTOR *V;
};


#endif // H_DS_VECTOR
