/** \file WO_Object.h
    Generic Object.

Copyright (c) 1998-1999 by Amir Geva.
This file is part of the Photon Game Development library,
beta release version 0.21.
Permission is granted to use and copy this file for non-commercial use only.  
Please contact the author concerning commercial usage. 
Amir Geva makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

*/
#ifndef H_WO_OBJECT
#define H_WO_OBJECT

#ifndef _CPPRTTI
#error Code must be compiled with RTTI  (Settings -> C++ -> Category: C++ Language -> Enable RTTI)
#endif 

#include <WO_Event.h>
#include <WO_Property.h>
#include <DS_Hashtable.h>
#include <DS_Queue.h>

class World;

/** Generic object class.  Can represent any entity in the program,
    and is not tied to any environment.
    All world objects should be subclasses of this one. */
class Object : public Integer_Containable
{
public:
  /** Construct an object.  Parent can be NULL, in which case it is set to the world object. */
  Object(World* W, Object* Parent);

  /** Destroys an object */
  virtual ~Object();

  /** Returns the object's ID. */
  virtual int  getID();

  /** Subclases must override this to supply rendering. */
  virtual long render();

  /** Subclasses may override this to provide state changes.
      Fraction gives the amount of time to provide change for (in seconds) */
  virtual long advance(float Fraction);

  /** Accepts events and places them in a queue. */
          long handleEvent(Event* E);

  /** These two methods allow access to the object's properties. */
          long getProperty(Property& P);

  /** Sets a propery.  Property object must be dynamically allocated */
          long setProperty(Property* P);

  /** Checks the event queue for events to process */
  long hasMoreEvents();

  /** Returns an event to process, without removing it from the queue */
  Event* peekEvent();

  /** Removes the next event from the queue */
  void   popEvent();


  /** Returns a current enumeration of all son objects. */
  virtual Enumeration objects();

  /** Returns non zero if object has son objects */
  virtual long hasSons();

  /** This method is called once before any rendering is done. */
  virtual long preRender();

  /** This method is called once per object before it is rendered.
      If it returns a non-zero value, the object will not be rendered. */
  virtual long preObjectRender(Object* O);

  /** This method is called once after all rendering is done. */
  virtual long postRender();

  /** This method is called once per object after it is rendered. */
  virtual long postObjectRender(Object* O);

  /** This method is called once before any object is advanced */
  virtual long preAdvance(float Fraction);

  /** This method is called once per object before it is advanced
      A non-zero return value means, that the object requests to be removed. */
  virtual long preObjectAdvance(Object* O, float Fraction);

  /** This method is called once after any object is advanced */
  virtual long postAdvance(float Fraction);

  /** This method is called once per object after it is advanced */
  virtual long postObjectAdvance(Object* O, float Fraction);


  /** Add an object to this parent.
      Object must be dynamically allocated
      method returns the object's ID. */
  virtual long add(Object* O);

  /** Remove this object.
      Object will be deleted automatically, by world at the end of advance */
  void        remove();

  /** Gets an object by its number (Use O->getID() to retrieve number)
      This is useful, instead of keeping pointers to objects, since
      objects may be deleted and then pointers are invalid.
      Both methods return NULL if object is not found.
      This method get an object which is a direct son of the current one. */
  virtual Object* getSon(int ID);

  /** Get the object's parent. Defaults to Object's World initially */
  virtual Object* getParent();

  /** This method gets an object anywhere in the system. */
  static Object* get(int ID);

  friend class World;
private:
  /** Removes this one from its parent */
  long removeNow();

  static Hashtable s_AllObjects;
  static Queue     s_Removals;

  Hashtable  m_Properties;
  Queue      m_Events;

protected:
  /** Sets the object's parent. Defaults to Object's World initially */
  virtual void setParent(Object* Parent);

  /** Removes a son.  Called by the world's postAdvance */
  virtual long removeNow(Object* O);

  World     *m_World;
  Object    *m_Parent;
  Vector    *m_Sons;
};

#endif // H_WO_OBJECT
